"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingdetails_output import BillingDetailsOutput, BillingDetailsOutputTypedDict
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class BuyerTypedDict(TypedDict):
    merchant_account_id: str
    r"""The ID of the merchant account this buyer belongs to."""
    created_at: datetime
    r"""The date this buyer was created at."""
    updated_at: datetime
    r"""The date this buyer was last updated at."""
    type: Literal["buyer"]
    r"""Always `buyer`."""
    id: NotRequired[Nullable[str]]
    r"""The ID for the buyer."""
    display_name: NotRequired[Nullable[str]]
    r"""The display name for the buyer."""
    external_identifier: NotRequired[Nullable[str]]
    r"""The merchant identifier for this buyer."""
    billing_details: NotRequired[Nullable[BillingDetailsOutputTypedDict]]
    r"""The billing name, address, email, and other fields for this buyer."""
    account_number: NotRequired[Nullable[str]]
    r"""The buyer account number"""


class Buyer(BaseModel):
    merchant_account_id: str
    r"""The ID of the merchant account this buyer belongs to."""

    created_at: datetime
    r"""The date this buyer was created at."""

    updated_at: datetime
    r"""The date this buyer was last updated at."""

    TYPE: Annotated[
        Annotated[Optional[Literal["buyer"]], AfterValidator(validate_const("buyer"))],
        pydantic.Field(alias="type"),
    ] = "buyer"
    r"""Always `buyer`."""

    id: OptionalNullable[str] = UNSET
    r"""The ID for the buyer."""

    display_name: OptionalNullable[str] = UNSET
    r"""The display name for the buyer."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""The merchant identifier for this buyer."""

    billing_details: OptionalNullable[BillingDetailsOutput] = UNSET
    r"""The billing name, address, email, and other fields for this buyer."""

    account_number: OptionalNullable[str] = UNSET
    r"""The buyer account number"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "id",
            "display_name",
            "external_identifier",
            "billing_details",
            "account_number",
        ]
        nullable_fields = [
            "id",
            "display_name",
            "external_identifier",
            "billing_details",
            "account_number",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


DefinitionFieldFormat = Union[
    Literal["text", "multiline", "number", "timezone", "boolean"], UnrecognizedStr
]
